package com.example.myinterface.myInterface;

import android.text.TextUtils;
import android.util.Log;

import java.util.HashMap;
import java.util.Map;

/**
 * Create by cwb on 2019/2/21
 */
public class FunctionManager {
    public static final String TAG = "FunctionManager";

    private Map<String, FunctionNoParamNoResult> noParamNoResultMap;
    private Map<String, FunctionHasParamNoResult> hasParamNoResultMap;
    private Map<String, FunctionNoParamHasResult> noParamHasResultMap;
    private Map<String, FunctionHasParamHasResult> hasParamHasResultMap;

    private static FunctionManager instance;

    private FunctionManager() {
        noParamHasResultMap = new HashMap<>();
        noParamNoResultMap = new HashMap<>();
        hasParamHasResultMap = new HashMap<>();
        hasParamNoResultMap = new HashMap<>();
    }

    public static FunctionManager getInstance() {
        if (instance == null) {
            synchronized (FunctionManager.class) {
                if (instance == null) {
                    instance = new FunctionManager();
                }
            }
        }
        return instance;
    }

    public void addFunction(FunctionNoParamNoResult functionNoParamNoResult){
        noParamNoResultMap.put(functionNoParamNoResult.functionName, functionNoParamNoResult);
    }

    public void addFunction(FunctionHasParamNoResult functionHasParamNoResult){
        hasParamNoResultMap.put(functionHasParamNoResult.functionName, functionHasParamNoResult);
    }

    public void addFunction(FunctionNoParamHasResult functionNoParamHasResult){
        noParamHasResultMap.put(functionNoParamHasResult.functionName, functionNoParamHasResult);
    }

    public void addFunction(FunctionHasParamHasResult functionHasParamHasResult){
        hasParamHasResultMap.put(functionHasParamHasResult.functionName, functionHasParamHasResult);
    }

    /**
     *  无参，无返回值
     */
    public void invokeNN(String functionName){
        if (TextUtils.isEmpty(functionName)){
            return;
        }
        if (noParamNoResultMap != null){
            FunctionNoParamNoResult functionNoParamNoResult = noParamNoResultMap.get(functionName);
            if (functionNoParamNoResult != null){
                functionNoParamNoResult.function();
            }else {
                Log.e(TAG, "未找到方法");
            }
        }
    }

    /**
     *  无参，有返回值
     */
    public <T> T invokeNH(String functionName, Class<T> t){
        if (TextUtils.isEmpty(functionName)){
            return null;
        }
        if (noParamHasResultMap != null){
            FunctionNoParamHasResult f = noParamHasResultMap.get(functionName);
            if (f != null){
                return t.cast(f.function());
            }else {
                Log.e(TAG, "未找到方法");
            }
        }
        return null;
    }

    /**
     * 有参，无返回值
     */
    public <P> void invokeHN(String functionName, P p){
        if (TextUtils.isEmpty(functionName)){
            return;
        }
        if (hasParamNoResultMap != null){
            FunctionHasParamNoResult f = hasParamNoResultMap.get(functionName);
            if (f != null){
                f.function(p);
            }else {
                Log.e(TAG, "未找到方法");
            }
        }
    }

    /**
     * 有参，有返回值
     */
    public <T, P> T invokeHH(String functionName, P p, Class<T> t){
        if (TextUtils.isEmpty(functionName)){
            return null;
        }
        if (hasParamHasResultMap != null){
            FunctionHasParamHasResult f = hasParamHasResultMap.get(functionName);
            if (f != null){
                if (t != null){
                    return t.cast(f.function(p));
                }
            }else {
                Log.e(TAG, "未找到方法");
            }
        }
        return null;
    }

    public void removeFunction(String functionName) {
        Map map = null;
        if (noParamNoResultMap != null){
            if (noParamNoResultMap.get(functionName) != null){
                map = noParamNoResultMap;
            }
        }
        if (noParamHasResultMap != null){
            if (noParamHasResultMap.get(functionName) != null){
                map = noParamHasResultMap;
            }
        }
        if (hasParamNoResultMap != null){
            if (hasParamNoResultMap.get(functionName) != null){
                map = hasParamNoResultMap;
            }
        }
        if (hasParamHasResultMap != null){
            if (hasParamHasResultMap.get(functionName) != null){
                map = hasParamHasResultMap;
            }
        }
        if (map != null){
            map.remove(functionName);
        }
    }

}
